@@ -2,6 +2,8 @@
 #define COLUMNDISPLAYFORMATDIALOG_H
 
 #include <QDialog>
+#include <QString>
+#include <QMap>
 
 namespace Ui {
 class ColumnDisplayFormatDialog;
@@ -12,7 +14,7 @@ class ColumnDisplayFormatDialog : public QDialog
     Q_OBJECT
 
 public:
-    explicit ColumnDisplayFormatDialog(const QString& colname, QString current_format, QWidget* parent = 0);
+    explicit ColumnDisplayFormatDialog(const QString& colname, QString current_format, QWidget* parent = nullptr);
     ~ColumnDisplayFormatDialog();
 
     QString selectedDisplayFormat() const;
@@ -23,6 +25,7 @@ private slots:
 private:
     Ui::ColumnDisplayFormatDialog* ui;
     QString column_name;
+    QMap<QString, QString> formatFunctions;
 };
 
 #endif
