@@ -5,6 +5,7 @@
 #include <QPersistentModelIndex>
 
 class QHexEdit;
+class DockTextEdit;
 
 namespace Ui {
 class EditDialog;
@@ -15,7 +16,7 @@ class EditDialog : public QDialog
     Q_OBJECT
 
 public:
-    explicit EditDialog(QWidget* parent = 0);
+    explicit EditDialog(QWidget* parent = nullptr);
     ~EditDialog();
 
     void setCurrentIndex(const QModelIndex& idx);
@@ -35,13 +36,16 @@ private slots:
     void setNull();
     void updateApplyButton();
     virtual void accept();
-    int checkDataType(const QByteArray& data);
     void loadData(const QByteArray& data);
     void toggleOverwriteMode();
     void editModeChanged(int newMode);
     void editTextChanged();
-    void updateCellInfo(const QByteArray& data);
-    QString humanReadableSize(double byteCount);
+    void switchEditorMode(bool autoSwitchForType);
+    void updateCellInfoAndMode(const QByteArray& data);
+    void setMustIndentAndCompact(bool enable);
+    void openPrintDialog();
+    void openPrintImageDialog();
+    void copyHexAscii();
 
 signals:
     void recordTextUpdated(const QPersistentModelIndex& idx, const QByteArray& data, bool isBlob);
@@ -49,29 +53,45 @@ signals:
 private:
     Ui::EditDialog* ui;
     QHexEdit* hexEdit;
+    DockTextEdit* sciEdit;
     QPersistentModelIndex currentIndex;
     int dataSource;
     int dataType;
-    bool textNullSet;
     bool isReadOnly;
+    bool mustIndentAndCompact;
+    QByteArray removedBom;
 
     enum DataSources {
         TextBuffer,
-        HexBuffer
+        HexBuffer,
+        SciBuffer
     };
 
+    // SVG is both an Image and an XML document so it is treated separately
     enum DataTypes {
         Binary,
         Image,
         Null,
-        Text
+        Text,
+        JSON,
+        SVG
     };
 
+    // Edit modes and editor stack (this must be aligned with the UI)
+    // Note that JSON and XML share the Scintilla widget.
     enum EditModes {
         TextEditor = 0,
         HexEditor = 1,
-        ImageViewer = 2
+        ImageViewer = 2,
+        JsonEditor, SciEditor = 3,
+        XmlEditor = 4
     };
+
+    int checkDataType(const QByteArray& data);
+    QString humanReadableSize(double byteCount) const;
+    bool promptInvalidData(const QString& dataType, const QString& errorString);
+    void setDataInBuffer(const QByteArray& data, DataSources source);
+    void setStackCurrentIndex(int editMode);
 };
 
 #endif
