@@ -11,8 +11,9 @@
 #include <QJsonDocument>
 #include <QJsonArray>
 #include <QJsonObject>
+#include <QTextCodec>
 
-ExportDataDialog::ExportDataDialog(DBBrowserDB& db, ExportFormats format, QWidget* parent, const QString& query, const QString& selection)
+ExportDataDialog::ExportDataDialog(DBBrowserDB& db, ExportFormats format, QWidget* parent, const QString& query, const sqlb::ObjectIdentifier& selection)
     : QDialog(parent),
       ui(new Ui::ExportDataDialog),
       pdb(db),
@@ -42,9 +43,17 @@ ExportDataDialog::ExportDataDialog(DBBrowserDB& db, ExportFormats format, QWidge
     if(query.isEmpty())
     {
         // Get list of tables to export
-        objectMap objects = pdb.getBrowsableObjects();
-        foreach(const sqlb::ObjectPtr& obj, objects)
-            ui->listTables->addItem(new QListWidgetItem(QIcon(QString(":icons/%1").arg(sqlb::Object::typeToString(obj->type()))), obj->name()));
+        for(auto it=pdb.schemata.constBegin();it!=pdb.schemata.constEnd();++it)
+        {
+            QList<sqlb::ObjectPtr> tables = it->values("table") + it->values("view");
+            for(auto jt=tables.constBegin();jt!=tables.constEnd();++jt)
+            {
+                sqlb::ObjectIdentifier obj(it.key(), (*jt)->name());
+                QListWidgetItem* item = new QListWidgetItem(QIcon(QString(":icons/%1").arg(sqlb::Object::typeToString((*jt)->type()))), obj.toDisplayString());
+                item->setData(Qt::UserRole, obj.toVariant());
+                ui->listTables->addItem(item);
+            }
+        }
 
         // Sort list of tables and select the table specified in the selection parameter or alternatively the first one
         ui->listTables->model()->sort(0);
@@ -52,9 +61,14 @@ ExportDataDialog::ExportDataDialog(DBBrowserDB& db, ExportFormats format, QWidge
         {
             ui->listTables->setCurrentItem(ui->listTables->item(0));
         } else {
-            QList<QListWidgetItem*> items = ui->listTables->findItems(selection, Qt::MatchExactly);
-            if(!items.isEmpty())
-                ui->listTables->setCurrentItem(items.first());
+            for(int i=0;i<ui->listTables->count();i++)
+            {
+                if(sqlb::ObjectIdentifier(ui->listTables->item(i)->data(Qt::UserRole)) == selection)
+                {
+                    ui->listTables->setCurrentRow(i);
+                    break;
+                }
+            }
         }
     } else {
         // Hide table combo box
@@ -103,7 +117,8 @@ bool ExportDataDialog::exportQueryCsv(const QString& sQuery, const QString& sFil
         QByteArray utf8Query = sQuery.toUtf8();
         sqlite3_stmt *stmt;
 
-        int status = sqlite3_prepare_v2(pdb._db, utf8Query.data(), utf8Query.size(), &stmt, NULL);
+        auto pDb = pdb.get(tr("exporting CSV"));
+        int status = sqlite3_prepare_v2(pDb.get(), utf8Query.data(), utf8Query.size(), &stmt, nullptr);
         if(SQLITE_OK == status)
         {
             if(ui->checkHeader->isChecked())
@@ -185,7 +200,9 @@ bool ExportDataDialog::exportQueryJson(const QString& sQuery, const QString& sFi
     {
         QByteArray utf8Query = sQuery.toUtf8();
         sqlite3_stmt *stmt;
-        int status = sqlite3_prepare_v2(pdb._db, utf8Query.data(), utf8Query.size(), &stmt, NULL);
+
+        auto pDb = pdb.get(tr("exporting JSON"));
+        int status = sqlite3_prepare_v2(pDb.get(), utf8Query.data(), utf8Query.size(), &stmt, nullptr);
 
         QJsonArray json_table;
 
@@ -207,10 +224,39 @@ bool ExportDataDialog::exportQueryJson(const QString& sQuery, const QString& sFi
                 QJsonObject json_row;
                 for(int i=0;i<columns;++i)
                 {
-                    QString content = QString::fromUtf8(
-                                (const char*)sqlite3_column_blob(stmt, i),
-                                sqlite3_column_bytes(stmt, i));
-                    json_row.insert(column_names[i], content);
+                    int type = sqlite3_column_type(stmt, i);
+
+                    switch (type) {
+                    case SQLITE_INTEGER: {
+                        qint64 content = sqlite3_column_int64(stmt, i);
+                        json_row.insert(column_names[i], content);
+                        break;
+                    }
+                    case SQLITE_FLOAT: {
+                        double content = sqlite3_column_double(stmt, i);
+                        json_row.insert(column_names[i], content);
+                        break;
+                    }
+                    case SQLITE_NULL: {
+                        json_row.insert(column_names[i], QJsonValue());
+                        break;
+                    }
+                    case SQLITE_TEXT: {
+                        QString content = QString::fromUtf8(
+                            (const char*)sqlite3_column_text(stmt, i),
+                            sqlite3_column_bytes(stmt, i));
+                        json_row.insert(column_names[i], content);
+                        break;
+                    }
+                    case SQLITE_BLOB: {
+                        QByteArray content((const char*)sqlite3_column_blob(stmt, i),
+                                           sqlite3_column_bytes(stmt, i));
+                        QTextCodec *codec = QTextCodec::codecForName("UTF-8");
+                        QString string = codec->toUnicode(content.toBase64(QByteArray::Base64Encoding));
+                        json_row.insert(column_names[i], string);
+                        break;
+                    }
+                    }
                 }
                 json_table.push_back(json_row);
 
@@ -311,7 +357,7 @@ void ExportDataDialog::accept()
                 return;
             }
 
-            foreach(QListWidgetItem* item, selectedItems)
+            for(const QListWidgetItem* item : selectedItems)
                 filenames << QDir(exportfolder).filePath(item->text() + default_file_extension);
         }
 
@@ -320,8 +366,7 @@ void ExportDataDialog::accept()
         {
             // if we are called from execute sql tab, query is already set
             // and we only export 1 select
-            QString sQuery = QString("SELECT * FROM %1;").arg(sqlb::escapeIdentifier(selectedItems.at(i)->text()));
-
+            QString sQuery = QString("SELECT * FROM %1;").arg(sqlb::ObjectIdentifier(selectedItems.at(i)->data(Qt::UserRole)).toString());
             exportQuery(sQuery, filenames.at(i));
         }
     }
