@@ -9,6 +9,7 @@ class QTreeWidgetItem;
 class QFrame;
 class QTableWidget;
 class QSslCertificate;
+class QAbstractButton;
 
 namespace Ui {
 class PreferencesDialog;
@@ -19,7 +20,7 @@ class PreferencesDialog : public QDialog
     Q_OBJECT
 
 public:
-    explicit PreferencesDialog(QWidget* parent = 0);
+    explicit PreferencesDialog(QWidget* parent = nullptr);
     ~PreferencesDialog();
 
 private slots:
@@ -34,12 +35,19 @@ private slots:
     virtual void addClientCertificate();
     virtual void removeClientCertificate();
     void chooseRemoteCloneDirectory();
+    void updatePreviewFont();
+
+    void on_buttonManageFileExtension_clicked();
+    void on_buttonBox_clicked(QAbstractButton* button);
 
 private:
     Ui::PreferencesDialog *ui;
 
+    QStringList m_dbFileExtensions;
+
     void fillLanguageBox();
     void loadColorSetting(QFrame *frame, const QString &name);
+    void setColorSetting(QFrame *frame, const QColor &color);
     void saveColorSetting(QFrame *frame, const QString &name);
     void addClientCertToTable(const QString& path, const QSslCertificate& cert);
 
