@@ -4,6 +4,7 @@
 #include <QDialog>
 
 class RemoteDatabase;
+class QRegExpValidator;
 
 namespace Ui {
 class RemotePushDialog;
@@ -22,6 +23,7 @@ public:
     QString licence() const;
     bool isPublic() const;
     QString branch() const;
+    bool forcePush() const;
 
 private:
     Ui::RemotePushDialog* ui;
@@ -33,6 +35,10 @@ private:
     // Reference to the remote database object which is stored somewhere in the main window
     RemoteDatabase& remoteDatabase;
 
+    // Validators
+    QRegExpValidator* m_nameValidator;
+    QRegExpValidator* m_branchValidator;
+
 protected slots:
     void checkInput();
     virtual void accept();
