@@ -6,7 +6,6 @@
 class SqlTextEdit;
 class SqliteTableModel;
 class DBBrowserDB;
-class QMenu;
 class ExtendedTableWidget;
 
 namespace Ui {
@@ -18,7 +17,7 @@ class SqlExecutionArea : public QWidget
     Q_OBJECT
 
 public:
-    explicit SqlExecutionArea(DBBrowserDB& _db, QWidget* parent = Q_NULLPTR);
+    explicit SqlExecutionArea(DBBrowserDB& _db, QWidget* parent = nullptr);
     ~SqlExecutionArea();
 
     QString getSql() const;
@@ -30,20 +29,31 @@ public:
     SqliteTableModel* getModel() { return model; }
     SqlTextEdit* getEditor();
     ExtendedTableWidget *getTableResult();
-    
+
 public slots:
-    virtual void finishExecution(const QString& result);
-    virtual void enableSaveButton(bool enable);
+    virtual void finishExecution(const QString& result, const bool ok);
     virtual void saveAsCsv();
-    virtual void saveAsView();
     virtual void reloadSettings();
+    void fetchedData();
+    void setFindFrameVisibility(bool show);
+
+private slots:
+    void findPrevious();
+    void findNext();
+    void findLineEdit_textChanged(const QString& text);
+    void hideFindFrame();
+
+signals:
+    void findFrameVisibilityChanged(bool visible);
 
 private:
+    void find(QString expr, bool forward);
     DBBrowserDB& db;
     SqliteTableModel* model;
-    QMenu* menuPopupSave;
     QString sqlFileName;
     Ui::SqlExecutionArea* ui;
+    bool m_columnsResized;              // This is set to true if the columns of the table view were already adjusted to fit their contents
+    bool showErrorIndicators;
 };
 
 #endif
