@@ -1,5 +1,9 @@
+#include <algorithm>
+
 #include "SqlUiLexer.h"
 #include "Qsci/qsciapis.h"
+#include "Settings.h"
+#include "sqlitetypes.h"
 
 SqlUiLexer::SqlUiLexer(QObject* parent) :
     QsciLexerSQL(parent)
@@ -29,7 +33,7 @@ void SqlUiLexer::setupAutoCompletion()
             << "DATABASE" << "DEFAULT" << "DEFERRABLE" << "DEFERRED" << "DELETE"
             << "DESC" << "DETACH" << "DISTINCT" << "DROP" << "EACH"
             << "ELSE" << "END" << "ESCAPE" << "EXCEPT" << "EXCLUSIVE"
-            << "EXISTS" << "EXPLAIN" << "FAIL" << "FOR" << "FOREIGN"
+            << "EXISTS" << "EXPLAIN" << "FAIL" << "FILTER" << "FOLLOWING" << "FOR" << "FOREIGN"
             << "FROM" << "FULL" << "GLOB" << "GROUP" << "HAVING"
             << "IF" << "IGNORE" << "IMMEDIATE" << "IN" << "INDEX"
             << "INDEXED" << "INITIALLY" << "INNER" << "INSERT" << "INSTEAD"
@@ -37,20 +41,23 @@ void SqlUiLexer::setupAutoCompletion()
             << "KEY" << "LEFT" << "LIKE" << "LIMIT" << "MATCH"
             << "NATURAL" << "NO" << "NOT" << "NOTNULL" << "NULL"
             << "OF" << "OFFSET" << "ON" << "OR" << "ORDER"
-            << "OUTER" << "PLAN" << "PRAGMA" << "PRIMARY" << "QUERY"
-            << "RAISE" << "RECURSIVE" << "REFERENCES" << "REGEXP" << "REINDEX" << "RELEASE"
+            << "OUTER" << "OVER" << "PARTITION" << "PLAN" << "PRAGMA" << "PRECEDING" << "PRIMARY" << "QUERY"
+            << "RAISE" << "RANGE" << "RECURSIVE" << "REFERENCES" << "REGEXP" << "REINDEX" << "RELEASE"
             << "RENAME" << "REPLACE" << "RESTRICT" << "RIGHT" << "ROLLBACK"
-            << "ROW" << "SAVEPOINT" << "SELECT" << "SET" << "TABLE"
+            << "ROWID" << "ROWS" << "SAVEPOINT" << "SELECT" << "SET" << "TABLE"
             << "TEMP" << "TEMPORARY" << "THEN" << "TO" << "TRANSACTION"
-            << "TRIGGER" << "UNION" << "UNIQUE" << "UPDATE" << "USING"
+            << "TRIGGER" << "UNBOUNDED" << "UNION" << "UNIQUE" << "UPDATE" << "USING"
             << "VACUUM" << "VALUES" << "VIEW" << "VIRTUAL" << "WHEN"
             << "WHERE" << "WITH" << "WITHOUT"
             // Data types
             << "INT" << "INTEGER" << "REAL" << "TEXT" << "BLOB" << "NUMERIC" << "CHAR";
-    foreach(const QString& keyword, keywordPatterns)
+    bool upperKeywords = Settings::getValue("editor", "upper_keywords").toBool();
+    for(const QString& keyword : keywordPatterns)
     {
-        autocompleteApi->add(keyword + "?" + QString::number(ApiCompleterIconIdKeyword));
-        autocompleteApi->add(keyword.toLower() + "?" + QString::number(ApiCompleterIconIdKeyword));
+        if (upperKeywords)
+            autocompleteApi->add(keyword + "?" + QString::number(ApiCompleterIconIdKeyword));
+        else
+            autocompleteApi->add(keyword.toLower() + "?" + QString::number(ApiCompleterIconIdKeyword));
     }
 
     // Functions
@@ -69,8 +76,8 @@ void SqlUiLexer::setupAutoCompletion()
             << "length" + tr("(X) For a string value X, the length(X) function returns the number of characters (not bytes) in X prior to the first NUL character.")
             << "like" + tr("(X,Y) The like() function is used to implement the \"Y LIKE X\" expression.")
             << "like" + tr("(X,Y,Z) The like() function is used to implement the \"Y LIKE X ESCAPE Z\" expression.")
-            << "load_extension" + tr("(X) The load_extension(X) function loads SQLite extensions out of the shared library file named X.")
-            << "load_extension" + tr("(X,Y) The load_extension(X) function loads SQLite extensions out of the shared library file named X using the entry point Y.")
+            << "load_extension" + tr("(X) The load_extension(X) function loads SQLite extensions out of the shared library file named X.\nUse of this function must be authorized from Preferences.")
+            << "load_extension" + tr("(X,Y) The load_extension(X) function loads SQLite extensions out of the shared library file named X using the entry point Y.\nUse of this function must be authorized from Preferences.")
             << "lower" + tr("(X) The lower(X) function returns a copy of string X with all ASCII characters converted to lower case.")
             << "ltrim" + tr("(X) ltrim(X) removes spaces from the left side of X.")
             << "ltrim" + tr("(X,Y) The ltrim(X,Y) function returns a string formed by removing any and all characters that appear in Y from the left side of X.")
@@ -110,10 +117,27 @@ void SqlUiLexer::setupAutoCompletion()
             << "max" + tr("(X) The max() aggregate function returns the maximum value of all values in the group.")
             << "min" + tr("(X) The min() aggregate function returns the minimum non-NULL value of all values in the group.")
             << "sum" + tr("(X) The sum() and total() aggregate functions return sum of all non-NULL values in the group.")
-            << "total" + tr("(X) The sum() and total() aggregate functions return sum of all non-NULL values in the group.");
+            << "total" + tr("(X) The sum() and total() aggregate functions return sum of all non-NULL values in the group.")
+            // Window functions
+            << "row_number" + tr("() The number of the row within the current partition. Rows are numbered starting from 1 in the order defined by the ORDER BY clause in the window definition, or in arbitrary order otherwise.")
+            << "rank" + tr("() The row_number() of the first peer in each group - the rank of the current row with gaps. If there is no ORDER BY clause, then all rows are considered peers and this function always returns 1.")
+            << "dense_rank" + tr("() The number of the current row's peer group within its partition - the rank of the current row without gaps. Partitions are numbered starting from 1 in the order defined by the ORDER BY clause in the window definition. If there is no ORDER BY clause, then all rows are considered peers and this function always returns 1. ")
+            << "percent_rank" + tr("() Despite the name, this function always returns a value between 0.0 and 1.0 equal to (rank - 1)/(partition-rows - 1), where rank is the value returned by built-in window function rank() and partition-rows is the total number of rows in the partition. If the partition contains only one row, this function returns 0.0. ")
+            << "cume_dist" + tr("() The cumulative distribution. Calculated as row-number/partition-rows, where row-number is the value returned by row_number() for the last peer in the group and partition-rows the number of rows in the partition.")
+            << "ntile" + tr("(N) Argument N is handled as an integer. This function divides the partition into N groups as evenly as possible and assigns an integer between 1 and N to each group, in the order defined by the ORDER BY clause, or in arbitrary order otherwise. If necessary, larger groups occur first. This function returns the integer value assigned to the group that the current row is a part of.")
+            << "lag" + tr("(expr) Returns the result of evaluating expression expr against the previous row in the partition. Or, if there is no previous row (because the current row is the first), NULL.")
+            << "lag" + tr("(expr,offset) If the offset argument is provided, then it must be a non-negative integer. In this case the value returned is the result of evaluating expr against the row offset rows before the current row within the partition. If offset is 0, then expr is evaluated against the current row. If there is no row offset rows before the current row, NULL is returned.")
+            << "lag" + tr("(expr,offset,default) If default is also provided, then it is returned instead of NULL if the row identified by offset does not exist.")
+            << "lead" + tr("(expr) Returns the result of evaluating expression expr against the next row in the partition. Or, if there is no next row (because the current row is the last), NULL.")
+            << "lead" + tr("(expr,offset) If the offset argument is provided, then it must be a non-negative integer. In this case the value returned is the result of evaluating expr against the row offset rows after the current row within the partition. If offset is 0, then expr is evaluated against the current row. If there is no row offset rows after the current row, NULL is returned.")
+            << "lead" + tr("(expr,offset,default) If default is also provided, then it is returned instead of NULL if the row identified by offset does not exist.")
+            << "first_value" + tr("(expr) This built-in window function calculates the window frame for each row in the same way as an aggregate window function. It returns the value of expr evaluated against the first row in the window frame for each row.")
+            << "last_value" + tr("(expr) This built-in window function calculates the window frame for each row in the same way as an aggregate window function. It returns the value of expr evaluated against the last row in the window frame for each row.")
+            << "nth_value" + tr("(expr,N) This built-in window function calculates the window frame for each row in the same way as an aggregate window function. It returns the value of expr evaluated against the row N of the window frame. Rows are numbered within the window frame starting from 1 in the order defined by the ORDER BY clause if one is present, or in arbitrary order otherwise. If there is no Nth row in the partition, then NULL is returned.")
+            ;
 
     listFunctions.clear();
-    foreach(const QString& keyword, functionPatterns)
+    for(const QString& keyword : functionPatterns)
     {
         QString fn = keyword.left(keyword.indexOf('('));
         QString descr = keyword.mid(keyword.indexOf('('));
@@ -126,20 +150,31 @@ void SqlUiLexer::setupAutoCompletion()
     }
 }
 
-void SqlUiLexer::setTableNames(const TablesAndColumnsMap& tables)
+void SqlUiLexer::setTableNames(const QualifiedTablesMap& tables)
 {
     // Update list for auto completion
     autocompleteApi->clear();
     listTables.clear();
     setupAutoCompletion();
-    for(auto it=tables.constBegin();it!=tables.constEnd();++it)
+    for(auto itSchemas=tables.constBegin();itSchemas!=tables.constEnd();++itSchemas)
     {
-        foreach(const QString& field, it.value())
-            autocompleteApi->add(it.key() + "?" + QString::number(SqlUiLexer::ApiCompleterIconIdTable) + "." +
-                                 field + "?" + QString::number(SqlUiLexer::ApiCompleterIconIdColumn));
+        for(auto itTables=itSchemas.value().constBegin();itTables!=itSchemas.value().constEnd();++itTables)
+        {
+            // Completion for schema.table
+            autocompleteApi->add(itSchemas.key() + "?" + QString::number(SqlUiLexer::ApiCompleterIconIdSchema) + "." +
+                                 itTables.key() + "?" + QString::number(SqlUiLexer::ApiCompleterIconIdTable));
 
-        // Store the table name list in order to highlight them in a different colour
-        listTables.append(it.key());
+            for(const QString& field : itTables.value()) {
+                // Completion for table.field
+                autocompleteApi->add(itTables.key() + "?" + QString::number(SqlUiLexer::ApiCompleterIconIdTable) + "." +
+                                     field + "?" + QString::number(SqlUiLexer::ApiCompleterIconIdColumn));
+
+                // Completion for isolated field
+                autocompleteApi->add(field + "?" + QString::number(SqlUiLexer::ApiCompleterIconIdColumn));
+            }
+            // Store the table name list in order to highlight them in a different colour
+            listTables.append(itTables.key());
+        }
     }
     autocompleteApi->prepare();
 }
@@ -169,9 +204,15 @@ QStringList SqlUiLexer::autoCompletionWordSeparators() const
 {
     // The only word separator for auto completion in SQL is "." as in "tablename.columnname".
     // Because this isn't implemented in the default QScintilla SQL lexer for some reason we add it here.
-
+    // We also need to consider quoted identifiers as in "tablename"."columnname" with whatever quote character
+    // is configured.
     QStringList wl;
-    wl << ".";
+
+    QString escapeSeparator = sqlb::escapeIdentifier(".");
+    // Case for non symetric quotes, e.g. "[.]" to "].["
+    std::reverse(escapeSeparator.begin(), escapeSeparator.end());
+
+    wl << "." << escapeSeparator;
     return wl;
 }
 
