@@ -12,7 +12,7 @@ class SqlUiLexer : public QsciLexerSQL
     Q_OBJECT
 
 public:
-    explicit SqlUiLexer(QObject *parent = 0);
+    explicit SqlUiLexer(QObject *parent = nullptr);
 
     enum ApiCompleterIconId
     {
@@ -20,10 +20,13 @@ public:
         ApiCompleterIconIdFunction,
         ApiCompleterIconIdTable,
         ApiCompleterIconIdColumn,
+        ApiCompleterIconIdSchema,
     };
 
     typedef QMap<QString, QList<QString> > TablesAndColumnsMap;
-    void setTableNames(const TablesAndColumnsMap& tables);
+    typedef QMap<QString, TablesAndColumnsMap > QualifiedTablesMap;
+
+    void setTableNames(const QualifiedTablesMap& tables);
 
     virtual const char* keywords(int set) const;
 
