@@ -14,7 +14,7 @@ class VacuumDialog : public QDialog
     Q_OBJECT
 
 public:
-    explicit VacuumDialog(DBBrowserDB* _db, QWidget* parent = 0);
+    explicit VacuumDialog(DBBrowserDB* _db, QWidget* parent = nullptr);
     ~VacuumDialog();
 
 private:
