@@ -230,31 +230,41 @@ bool DBBrowserDB::attach(const QString& filePath, QString attach_as)
     QString key;
     if(cipherSettings && is_encrypted)
         key = "KEY " + cipherSettings->getPassword();
+    else
+        key = "KEY ''";
+
+    // Only apply cipher settings if the database is encrypted
+    if(cipherSettings && is_encrypted)
+    {
+        if(!executeSQL(QString("PRAGMA cipher_default_page_size = %1").arg(cipherSettings->getPageSize()), false))
+        {
+            QMessageBox::warning(nullptr, qApp->applicationName(), lastErrorMessage);
+            return false;
+        }
+        if(!executeSQL(QString("PRAGMA cipher_default_kdf_iter = %1").arg(cipherSettings->getKdfIterations()), false))
+        {
+            QMessageBox::warning(nullptr, qApp->applicationName(), lastErrorMessage);
+            return false;
+        }
+        if(!executeSQL(QString("PRAGMA cipher_hmac_algorithm = %1").arg(cipherSettings->getHmacAlgorithm()), false))
+        {
+            QMessageBox::warning(nullptr, qApp->applicationName(), lastErrorMessage);
+            return false;
+        }
+        if(!executeSQL(QString("PRAGMA cipher_kdf_algorithm = %1").arg(cipherSettings->getKdfAlgorithm()), false))
+        {
+            QMessageBox::warning(nullptr, qApp->applicationName(), lastErrorMessage);
+            return false;
+        }
+    }
+
     if(!executeSQL(QString("ATTACH '%1' AS %2 %3").arg(filePath).arg(sqlb::escapeIdentifier(attach_as)).arg(key), false))
     {
         QMessageBox::warning(nullptr, qApp->applicationName(), lastErrorMessage);
         return false;
     }
-    if(!executeSQL(QString("PRAGMA %1.cipher_page_size = %2").arg(sqlb::escapeIdentifier(attach_as)).arg(cipherSettings->getPageSize()), false))
-    {
-        QMessageBox::warning(nullptr, qApp->applicationName(), lastErrorMessage);
-        return false;
-    }
-    if(!executeSQL(QString("PRAGMA %1.kdf_iter = %2").arg(sqlb::escapeIdentifier(attach_as)).arg(cipherSettings->getKdfIterations()), false))
-    {
-        QMessageBox::warning(nullptr, qApp->applicationName(), lastErrorMessage);
-        return false;
-    }
-    if(!executeSQL(QString("PRAGMA %1.cipher_hmac_algorithm = %2").arg(sqlb::escapeIdentifier(attach_as)).arg(cipherSettings->getHmacAlgorithm()), false))
-    {
-        QMessageBox::warning(nullptr, qApp->applicationName(), lastErrorMessage);
-        return false;
-    }
-    if(!executeSQL(QString("PRAGMA %1.cipher_kdf_algorithm = %2").arg(sqlb::escapeIdentifier(attach_as)).arg(cipherSettings->getKdfAlgorithm()), false))
-    {
-        QMessageBox::warning(nullptr, qApp->applicationName(), lastErrorMessage);
-        return false;
-    }
+
+    // Clean up cipher settings
     delete cipherSettings;
 #else
     // Attach database
@@ -921,32 +931,10 @@ bool DBBrowserDB::executeMultiSQL(QByteArray query, bool dirty, bool log)
         return false;
     }
 
-    // Check if this SQL containts any transaction statements. If so remove them and create a savepoint instead by overriding the dirty parameter.
-    // TODO This should check for 'END TRANSACTION' too. It should be case insensitive and it should work with any amounts of whitespace etc. "Good" news
-    // is that none of this was ever done correctly before.
-    if(query.contains("BEGIN TRANSACTION;"))
-    {
-        query.replace("BEGIN TRANSACTION;", "                  ");
-        dirty = true;
-    }
-    if(query.contains("COMMIT;"))
-    {
-        query.replace("COMMIT;", "       ");
-        dirty = true;
-    }
-
     // Log the statement if needed
     if(log)
         logSQL(query, kLogMsg_App);
 
-    // Set DB to dirty/create restore point if necessary
-    QString savepoint_name;
-    if(dirty)
-    {
-        savepoint_name = generateSavepointName("execmultisql");
-        setSavepoint(savepoint_name);
-    }
-
     // Show progress dialog
     QProgressDialog progress(tr("Executing SQL..."),
                              tr("Cancel"), 0, 100);
@@ -963,6 +951,7 @@ bool DBBrowserDB::executeMultiSQL(QByteArray query, bool dirty, bool log)
     unsigned int line = 0;
     bool structure_updated = false;
     int last_progress_value = -1;
+    QString savepoint_name;
     while(tail && *tail != 0 && (res == SQLITE_OK || res == SQLITE_DONE))
     {
         line++;
@@ -981,8 +970,7 @@ bool DBBrowserDB::executeMultiSQL(QByteArray query, bool dirty, bool log)
             last_progress_value = progress_value;
         }
 
-        // Check whether the DB structure is changed by this statement
-        if(!dontCheckForStructureUpdates && !structure_updated)
+        // Check next statement
         {
             // Ignore all whitespace at the start of the current tail
             const char* tail_ptr = tail;
@@ -995,12 +983,39 @@ bool DBBrowserDB::executeMultiSQL(QByteArray query, bool dirty, bool log)
             std::string next_statement(tail_ptr, length);
             std::transform(next_statement.begin(), next_statement.end(), next_statement.begin(), ::toupper);
 
-            // Check if it's a modifying statement
-            if(next_statement.compare(0, 5, "ALTER") == 0 ||
-                    next_statement.compare(0, 6, "CREATE") == 0 ||
-                    next_statement.compare(0, 4, "DROP") == 0 ||
-                    next_statement.compare(0, 8, "ROLLBACK") == 0)
-                structure_updated = true;
+            // Check for transaction statements and skip until the next semicolon
+            if(next_statement.compare(0, 6, "COMMIT") == 0 ||
+                    next_statement.compare(0, 4, "END ") == 0 ||
+                    next_statement.compare(0, 6, "BEGIN ") == 0)
+            {
+                while(tail != tail_end)
+                {
+                    if(*tail++ == ';')
+                        break;
+                }
+
+                // Set DB to dirty and create a restore point if we haven't done that yet
+                if(savepoint_name.isNull())
+                {
+                    savepoint_name = generateSavepointName("execmultisql");
+                    setSavepoint(savepoint_name);
+                    dirty = true;
+                }
+
+                // Don't just execute next statement. Start next statement with the same checks
+                continue;
+            }
+
+            // Check whether the DB structure is changed by this statement
+            if(!dontCheckForStructureUpdates && !structure_updated)
+            {
+                // Check if it's a modifying statement
+                if(next_statement.compare(0, 5, "ALTER") == 0 ||
+                        next_statement.compare(0, 6, "CREATE") == 0 ||
+                        next_statement.compare(0, 4, "DROP") == 0 ||
+                        next_statement.compare(0, 8, "ROLLBACK") == 0)
+                    structure_updated = true;
+            }
         }
 
         // Execute next statement
