@@ -3,25 +3,75 @@
 #include "grammar/Sqlite3Parser.hpp"
 
 #include <sstream>
-#include <QDebug>
+#include <iostream>
+#include <clocale>          // This include seems to only be necessary for the Windows build
 
 namespace sqlb {
 
-QStringList Field::Datatypes = QStringList() << "INTEGER" << "TEXT" << "BLOB" << "REAL" << "NUMERIC";
+static escapeQuoting customQuoting = DoubleQuotes;
+
+void setIdentifierQuoting(escapeQuoting toQuoting)
+{
+    customQuoting = toQuoting;
+}
 
 QString escapeIdentifier(QString id)
 {
-    return '`' + id.replace('`', "``") + '`';
+    switch(customQuoting) {
+    case GraveAccents:
+        return '`' + id.replace('`', "``") + '`';
+    case SquareBrackets:
+        // There aren't any escaping possibilities for square brackets inside the identifier,
+        // so we rely on the user to not enter these characters when this kind of quoting is
+        // selected.
+        return '[' + id + ']';
+    case DoubleQuotes:
+    default:
+        // This may produce a 'control reaches end of non-void function' warning if the
+        // default branch is removed, even though we have covered all possibilities in the
+        // switch statement.
+        return '"' + id.replace('"', "\"\"") + '"';
+    }
 }
 
-QStringList fieldVectorToFieldNames(const FieldVector& vector)
+QStringList escapeIdentifier(const QStringList& ids)
 {
-    QStringList result;
-    foreach(const FieldPtr& field, vector)
-        result.append(escapeIdentifier(field->name()));
-    return result;
+    QStringList ret;
+    for(const QString& id : ids)
+        ret.push_back(escapeIdentifier(id));
+    return ret;
 }
 
+/**
+ * @brief The SetLocaleToC class
+ * This is a stupid helper class which sets the current locale as used by the C++ standard library to the C locale.
+ * Upon destruction it resets it to whatever the previous locale was. This is used to work around a problem in Antlr's
+ * string comparison which because it is case-independent relies on the current locale. However, when parsind SQL
+ * statements we don't want the locale to interfere here. Especially the Turkish locale is problematic here because
+ * of the dotted I problem.
+ */
+class SetLocaleToC
+{
+public:
+    SetLocaleToC()
+    {
+        // Query current locale and save it
+        oldLocale = std::setlocale(LC_CTYPE, nullptr);
+
+        // Set locale for standard library functions
+        std::setlocale(LC_CTYPE, "C.UTF-8");
+    }
+
+    ~SetLocaleToC()
+    {
+        // Reset old locale
+        std::setlocale(LC_CTYPE, oldLocale.c_str());
+    }
+
+private:
+    std::string oldLocale;
+};
+
 /**
  * @brief The CreateTableWalker class
  * Goes trough the createtable AST and returns
@@ -38,6 +88,7 @@ public:
 
 private:
     void parsecolumn(Table* table, antlr::RefAST c);
+    QString parseConflictClause(antlr::RefAST c);
 
 private:
     antlr::RefAST m_root;
@@ -64,33 +115,6 @@ private:
     antlr::RefAST m_root;
 };
 
-ObjectPtr Object::parseSQL(Object::Types type, const QString& sSQL)
-{
-    // Parse SQL statement according to type
-    ObjectPtr result;
-    switch(type)
-    {
-    case Object::Types::Table:
-        result = Table::parseSQL(sSQL);
-        break;
-    case Object::Types::Index:
-        result = Index::parseSQL(sSQL);
-        break;
-    case Object::Types::View:
-        result = View::parseSQL(sSQL);
-        break;
-    case Object::Types::Trigger:
-        result = Trigger::parseSQL(sSQL);
-        break;
-    default:
-        return ObjectPtr(nullptr);
-    }
-
-    // Strore the original SQL statement and return the result
-    result->setOriginalSql(sSQL);
-    return result;
-}
-
 QString Object::typeToString(Types type)
 {
     switch(type)
@@ -119,13 +143,7 @@ QString ForeignKeyClause::toString() const
     QString result = escapeIdentifier(m_table);
 
     if(m_columns.size())
-    {
-        result += "(";
-        foreach(const QString& column, m_columns)
-            result += escapeIdentifier(column) + ',';
-        result.chop(1); // Remove last comma
-        result += ")";
-    }
+        result += "(" + escapeIdentifier(m_columns).join(",") + ")";
 
     if(m_constraint.size())
         result += " " + m_constraint;
@@ -138,37 +156,40 @@ void ForeignKeyClause::setFromString(const QString& fk)
     m_override = fk;
 }
 
-QString ForeignKeyClause::toSql(const FieldVector& applyOn) const
+QString ForeignKeyClause::toSql(const QStringList& applyOn) const
 {
     QString result;
     if(!m_name.isNull())
         result += QString("CONSTRAINT %1 ").arg(escapeIdentifier(m_name));
-    result += QString("FOREIGN KEY(%1) REFERENCES %2").arg(fieldVectorToFieldNames(applyOn).join(",")).arg(this->toString());
+    result += QString("FOREIGN KEY(%1) REFERENCES %2").arg(escapeIdentifier(applyOn).join(",")).arg(this->toString());
 
     return result;
 }
 
-QString UniqueConstraint::toSql(const FieldVector& applyOn) const
+QString UniqueConstraint::toSql(const QStringList& applyOn) const
 {
     QString result;
     if(!m_name.isNull())
         result += QString("CONSTRAINT %1 ").arg(escapeIdentifier(m_name));
-    result += QString("UNIQUE(%1)").arg(fieldVectorToFieldNames(applyOn).join(","));
+    result += QString("UNIQUE(%1)").arg(escapeIdentifier(applyOn).join(","));
 
     return result;
 }
 
-QString PrimaryKeyConstraint::toSql(const FieldVector& applyOn) const
+QString PrimaryKeyConstraint::toSql(const QStringList& applyOn) const
 {
     QString result;
     if(!m_name.isNull())
         result += QString("CONSTRAINT %1 ").arg(escapeIdentifier(m_name));
-    result += QString("PRIMARY KEY(%1)").arg(fieldVectorToFieldNames(applyOn).join(","));
+    result += QString("PRIMARY KEY(%1)").arg(escapeIdentifier(applyOn).join(","));
+
+    if(!m_conflictAction.isEmpty())
+        result += " ON CONFLICT " + m_conflictAction;
 
     return result;
 }
 
-QString CheckConstraint::toSql(const FieldVector&) const
+QString CheckConstraint::toSql(const QStringList&) const
 {
     QString result;
     if(!m_name.isNull())
@@ -225,17 +246,50 @@ bool Field::isInteger() const
             || norm == "int8";
 }
 
-void Table::clear()
+bool Field::isReal() const
 {
-    m_rowidColumn = "_rowid_";
-    m_fields.clear();
-    m_constraints.clear();
-    m_virtual = QString();
-    m_temporary = false;
+    QString norm = m_type.trimmed().toLower();
+
+    return     norm == "real"
+            || norm == "double"
+            || norm == "double precision"
+            || norm == "float";
 }
+
+bool Field::isNumeric() const
+{
+    QString norm = m_type.trimmed().toLower();
+
+    return     norm.startsWith("decimal")
+            || norm == "numeric"
+            || norm == "boolean"
+            || norm == "date"
+            || norm == "datetime";
+}
+
+bool Field::isBlob() const
+{
+    QString norm = m_type.trimmed().toLower();
+
+    return     norm.isEmpty()
+            || norm == "blob";
+}
+
+QString Field::affinity() const
+{
+    if (isInteger()) return "INTEGER";
+
+    if (isText()) return "TEXT";
+
+    if (isBlob()) return "BLOB";
+
+    if (isReal()) return "REAL";
+
+    return "NUMERIC";
+}
+
 Table::~Table()
 {
-    clear();
 }
 
 Table& Table::operator=(const Table& rhs)
@@ -248,123 +302,35 @@ Table& Table::operator=(const Table& rhs)
     m_virtual = rhs.m_virtual;
 
     // Clear the fields and the constraints first in order to avoid duplicates and/or old data in the next step
-    m_fields.clear();
+    fields.clear();
     m_constraints.clear();
 
     // Make copies of the fields and the constraints. This is necessary in order to avoid any unwanted changes to the application's main database
-    // schema representation just modifying a reference to the fields or constraints and thinking it operates on a copy.
-    foreach(FieldPtr f, rhs.m_fields)
-        addField(FieldPtr(new Field(*f)));
-    for(auto it=rhs.m_constraints.constBegin();it!=rhs.m_constraints.constEnd();++it)   // TODO This is so ugly, it should be replaced really by anything else
-    {
-        FieldVector key;
-        ConstraintPtr constraint;
-        foreach(FieldPtr f, it.key())
-            key.push_back(m_fields.at(findField(f->name())));
-        if(it.value()->type() == Constraint::ConstraintTypes::PrimaryKeyConstraintType)
-            constraint = ConstraintPtr(new PrimaryKeyConstraint(*(it.value().dynamicCast<PrimaryKeyConstraint>())));
-        else if(it.value()->type() == Constraint::ConstraintTypes::UniqueConstraintType)
-            constraint = ConstraintPtr(new UniqueConstraint(*(it.value().dynamicCast<UniqueConstraint>())));
-        else if(it.value()->type() == Constraint::ConstraintTypes::ForeignKeyConstraintType)
-            constraint = ConstraintPtr(new ForeignKeyClause(*(it.value().dynamicCast<ForeignKeyClause>())));
-        else if(it.value()->type() == Constraint::ConstraintTypes::CheckConstraintType)
-            constraint = ConstraintPtr(new CheckConstraint(*(it.value().dynamicCast<CheckConstraint>())));
-        else
-            qWarning() << "Unknown constraint type";
-        addConstraint(key, constraint);
-    }
+    // schema representation just by modifying a reference to the fields or constraints and thinking it operates on a copy.
+    for(const Field& f : rhs.fields)
+        fields.push_back(f);
+    m_constraints = rhs.m_constraints;
 
     return *this;
 }
 
-void Table::addField(const FieldPtr& f)
-{
-    m_fields.append(FieldPtr(f));
-}
-
-bool Table::removeField(const QString& sFieldName)
-{
-    int index = findField(sFieldName);
-    if( index != -1)
-    {
-        m_fields.remove(index);
-        return true;
-    }
-    return false;
-}
-
-void Table::setFields(const FieldVector &fields)
-{
-    clear();
-    m_fields = fields;
-}
-
-void Table::setField(int index, FieldPtr f)
-{
-    FieldPtr oldField = m_fields[index];
-    m_fields[index] = f;
-
-    // Update all constraints. If an existing field is updated but was used in a constraint, the pointers in the constraint key needs to be updated
-    // to the new field, too.
-    if(oldField)
-    {
-        ConstraintMap::iterator it = m_constraints.begin();
-        while(it != m_constraints.end())
-        {
-            // Loop through all fields mentioned in a foreign key
-            FieldVector fields = it.key();
-            bool modified = false;
-            for(int i=0;i<fields.size();++i)
-            {
-                // If the field that is being modified is in there update it to the new field and set a flag that something has changed.
-                // This is used below to know when to update the map key
-                if(fields[i] == oldField)
-                {
-                    fields[i] = f;
-                    modified = true;
-                }
-            }
-            if(modified)
-            {
-                // When we need to update the map key, we insert a new constraint using the updated field vector and the old
-                // constraint information, and delete the old one afterwards
-                m_constraints.insert(fields, it.value());
-                it = m_constraints.erase(it);
-            } else {
-                ++it;
-            }
-        }
-    }
-}
-
-int Table::findField(const QString &sname) const
-{
-    for(int i = 0; i < m_fields.count(); ++i)
-    {
-        if(m_fields.at(i)->name().compare(sname, Qt::CaseInsensitive) == 0)
-            return i;
-    }
-    return -1;
-}
-
-int Table::findPk() const
+Table::field_iterator Table::findPk()
 {
     // TODO This is a stupid function (and always was) which should be fixed/improved
 
-    FieldVector pk = primaryKey();
+    QStringList pk = primaryKey();
     if(pk.empty())
-        return -1;
+        return fields.end();
     else
-        return findField(pk.at(0)->name());
+        return findField(this, pk.at(0));
 }
 
 QStringList Table::fieldList() const
 {
     QStringList sl;
 
-    foreach(const FieldPtr& f, m_fields) {
-        sl << f->toString();
-    }
+    for(const Field& f : fields)
+        sl << f.toString();
 
     return sl;
 }
@@ -373,8 +339,8 @@ QStringList Table::fieldNames() const
 {
     QStringList sl;
 
-    foreach(FieldPtr f, m_fields)
-        sl << f->name();
+    for(const Field& f : fields)
+        sl << f.name();
 
     return sl;
 }
@@ -382,22 +348,24 @@ QStringList Table::fieldNames() const
 FieldInfoList Table::fieldInformation() const
 {
     FieldInfoList result;
-    foreach(FieldPtr f, m_fields)
-        result.append({f->name(), f->type(), f->toString("  ", " ")});
+    for(const Field& f : fields)
+        result.emplace_back(f.name(), f.type(), f.toString("  ", " "));
     return result;
 }
 
 bool Table::hasAutoIncrement() const
 {
-    foreach(FieldPtr f, m_fields) {
-        if(f->autoIncrement())
+    for(const Field& f : fields) {
+        if(f.autoIncrement())
             return true;
     }
     return false;
 }
 
-ObjectPtr Table::parseSQL(const QString &sSQL)
+TablePtr Table::parseSQL(const QString& sSQL)
 {
+    SetLocaleToC locale;
+
     std::stringstream s;
     s << sSQL.toStdString();
     Sqlite3Lexer lex(s);
@@ -413,47 +381,48 @@ ObjectPtr Table::parseSQL(const QString &sSQL)
         parser.createtable();
         CreateTableWalker ctw(parser.getAST());
 
-        return ctw.table();
+        auto t = ctw.table();
+        t->setOriginalSql(sSQL);
+        return t;
     }
     catch(antlr::ANTLRException& ex)
     {
-        qCritical() << "Sqlite parse error: " << QString::fromStdString(ex.toString()) << "(" << sSQL << ")";
+        std::cerr << "Sqlite parse error: " << ex.toString() << "(" << sSQL.toStdString() << ")" << std::endl;
     }
     catch(...)
     {
-        qCritical() << "Sqlite parse error: " << sSQL; //TODO
+        std::cerr << "Sqlite parse error: " << sSQL.toStdString() << std::endl; //TODO
     }
 
     return TablePtr(new Table(""));
 }
 
-QString Table::sql(bool ifNotExists) const
+QString Table::sql(const QString& schema, bool ifNotExists) const
 {
     // Special handling for virtual tables: just build an easy create statement and copy the using part in there
     if(isVirtual())
-        return QString("CREATE VIRTUAL TABLE %1 USING %2;").arg(escapeIdentifier(m_name)).arg(m_virtual);
+        return QString("CREATE VIRTUAL TABLE %1 USING %2;").arg(ObjectIdentifier(schema, m_name).toString(true)).arg(m_virtual);
 
     // This is a normal table, not a virtual one
-    QString sql = QString("CREATE %1TABLE%2 %3 (\n")
-            .arg(m_temporary ? QString("TEMPORARY ") : QString(""))
+    QString sql = QString("CREATE TABLE%1 %2 (\n")
             .arg(ifNotExists ? QString(" IF NOT EXISTS") : QString(""))
-            .arg(escapeIdentifier(m_name));
+            .arg(ObjectIdentifier(schema, m_name).toString(true));
 
     sql += fieldList().join(",\n");
 
     // Constraints
-    ConstraintMap::const_iterator it = m_constraints.constBegin();
+    ConstraintMap::const_iterator it = m_constraints.cbegin();
     bool autoincrement = hasAutoIncrement();
-    while(it != m_constraints.constEnd())
+    while(it != m_constraints.cend())
     {
         // Ignore auto increment primary key constraint
-        if((!autoincrement || it.value()->type() != Constraint::PrimaryKeyConstraintType))
+        if((!autoincrement || it->second->type() != Constraint::PrimaryKeyConstraintType))
         {
             // Ignore all constraints without any fields, except for check constraints which don't rely on a field vector
-            if(!(it.key().isEmpty() && it.value()->type() != Constraint::CheckConstraintType))
+            if(!(it->first.empty() && it->second->type() != Constraint::CheckConstraintType))
             {
                 sql += QString(",\n\t");
-                sql += it.value()->toSql(it.key());
+                sql += it->second->toSql(it->first);
             }
         }
         ++it;
@@ -468,12 +437,12 @@ QString Table::sql(bool ifNotExists) const
     return sql + ";";
 }
 
-void Table::addConstraint(FieldVector fields, ConstraintPtr constraint)
+void Table::addConstraint(QStringList fields, ConstraintPtr constraint)
 {
-    m_constraints.insert(fields, constraint);
+    m_constraints.insert({fields, constraint});
 }
 
-void Table::setConstraint(FieldVector fields, ConstraintPtr constraint)
+void Table::setConstraint(QStringList fields, ConstraintPtr constraint)
 {
     // Delete any old constraints of this type for these fields
     removeConstraints(fields, constraint->type());
@@ -482,38 +451,46 @@ void Table::setConstraint(FieldVector fields, ConstraintPtr constraint)
     addConstraint(fields, constraint);
 }
 
-void Table::removeConstraints(FieldVector fields, Constraint::ConstraintTypes type)
+void Table::removeConstraints(QStringList fields, Constraint::ConstraintTypes type)
 {
-    QList<ConstraintPtr> list = constraints(fields, type);
-
-    for (ConstraintPtr c : list) {
-        m_constraints.remove(fields, c);
+    for(auto it = m_constraints.begin();it!=m_constraints.end();)
+    {
+        if(it->first == fields && it->second->type() == type)
+            m_constraints.erase(it++);
+        else
+            ++it;
     }
 }
 
-ConstraintPtr Table::constraint(FieldVector fields, Constraint::ConstraintTypes type) const
+ConstraintPtr Table::constraint(QStringList fields, Constraint::ConstraintTypes type) const
 {
-    QList<ConstraintPtr> list = constraints(fields, type);
+    auto list = constraints(fields, type);
     if(list.size())
         return list.at(0);
     else
         return ConstraintPtr(nullptr);
 }
 
-QList<ConstraintPtr> Table::constraints(FieldVector fields, Constraint::ConstraintTypes type) const
+std::vector<ConstraintPtr> Table::constraints(QStringList fields, Constraint::ConstraintTypes type) const
 {
-    QList<ConstraintPtr> clist;
-    if(fields.isEmpty())
-        clist = m_constraints.values();
-    else
-        clist = m_constraints.values(fields);
+    ConstraintMap::const_iterator begin, end;
+    if(fields.empty())
+    {
+        begin = m_constraints.begin();
+        end = m_constraints.end();
+    } else {
+        std::tie(begin, end) = m_constraints.equal_range(fields);
+    }
+
+    std::vector<ConstraintPtr> clist;
+    std::transform(begin, end, std::back_inserter(clist), [](std::pair<QStringList, ConstraintPtr> elem){return elem.second;});
 
     if(type == Constraint::NoType)
     {
         return clist;
     } else {
-        QList<ConstraintPtr> clist_typed;
-        foreach(const ConstraintPtr& ptr, clist)
+        std::vector<ConstraintPtr> clist_typed;
+        for(const ConstraintPtr& ptr : clist)
         {
             if(ptr->type() == type)
                 clist_typed.push_back(ptr);
@@ -527,25 +504,66 @@ void Table::setConstraints(const ConstraintMap& constraints)
     m_constraints = constraints;
 }
 
-FieldVector& Table::primaryKeyRef()
+QStringList& Table::primaryKeyRef()
 {
-    return const_cast<FieldVector&>(static_cast<const Table*>(this)->primaryKey());
+    return const_cast<QStringList&>(static_cast<const Table*>(this)->primaryKey());
 }
 
-const FieldVector& Table::primaryKey() const
+const QStringList& Table::primaryKey() const
 {
-    auto it = m_constraints.constBegin();
-    while(it != m_constraints.constEnd())
+    auto it = m_constraints.cbegin();
+    while(it != m_constraints.cend())
     {
-        if(it.value()->type() == Constraint::PrimaryKeyConstraintType)
-            return it.key();
+        if(it->second->type() == Constraint::PrimaryKeyConstraintType)
+            return it->first;
         ++it;
     }
 
-    static FieldVector emptyFieldVector;
+    static QStringList emptyFieldVector;
     return emptyFieldVector;
 }
 
+void Table::removeKeyFromAllConstraints(const QString& key)
+{
+    // First remove all constraints with exactly that one key
+    m_constraints.erase({key});
+
+    // Then delete all occurrences of the key in compound columns
+    for(auto it=m_constraints.begin();it!=m_constraints.end();)
+    {
+        if(it->first.contains(key))
+        {
+            QStringList k = it->first;
+            k.removeAll(key);
+            m_constraints.insert({k, it->second});
+            it = m_constraints.erase(it);
+        } else {
+            ++it;
+        }
+    }
+}
+
+void Table::renameKeyInAllConstraints(const QString& key, const QString& to)
+{
+    // Do nothing if the key hasn't really changed
+    if(key == to)
+        return;
+
+    // Find all occurrences of the key and change it to the new one
+    for(auto it=m_constraints.begin();it!=m_constraints.end();)
+    {
+        if(it->first.contains(key))
+        {
+            QStringList k = it->first;
+            k.replaceInStrings(QRegExp("^" + key + "$"), to);
+            m_constraints.insert({k, it->second});
+            it = m_constraints.erase(it);
+        } else {
+            ++it;
+        }
+    }
+}
+
 namespace
 {
 QString identifier(antlr::RefAST ident)
@@ -592,6 +610,47 @@ QString concatTextAST(antlr::RefAST t, bool withspace = false)
     }
     return stext.join(withspace ? " " : "");
 }
+
+QString concatExprAST(antlr::RefAST t)
+{
+    QString expr;
+
+    int num_paren = 1;
+    while(t)
+    {
+        if(t->getType() == sqlite3TokenTypes::LPAREN)
+            num_paren++;
+        else if(t->getType() == sqlite3TokenTypes::RPAREN)
+            num_paren--;
+
+        if(num_paren == 0)
+            break;
+
+        switch(t->getType())
+        {
+        case sqlite3TokenTypes::AND:
+        case sqlite3TokenTypes::OR:
+        case sqlite3TokenTypes::IN:
+        case sqlite3TokenTypes::LIKE:
+        case sqlite3TokenTypes::MATCH:
+        case sqlite3TokenTypes::REGEXP:
+        case sqlite3TokenTypes::EXISTS:
+        case sqlite3TokenTypes::GLOB:
+        case sqlite3TokenTypes::BETWEEN:
+            expr.append(" " + textAST(t) + " ");
+            break;
+        case sqlite3TokenTypes::NOT:
+            expr.append(" " + textAST(t));
+            break;
+        default:
+            expr.append(textAST(t));
+        }
+
+        t = t->getNextSibling();
+    }
+
+    return expr.trimmed();
+}
 }
 
 namespace {
@@ -623,7 +682,7 @@ TablePtr CreateTableWalker::table()
         antlr::RefAST s = m_root->getFirstChild();
 
         // If the primary tree isn't filled, this isn't a normal CREATE TABLE statement. Switch to the next alternative tree.
-        if(s == 0)
+        if(s == nullptr)
             s = m_root->getNextSibling();
 
         // Skip to table name
@@ -700,14 +759,13 @@ TablePtr CreateTableWalker::table()
                     tc = tc->getNextSibling()->getNextSibling(); // skip primary and key
                     tc = tc->getNextSibling(); // skip LPAREN
 
-                    FieldVector fields;
+                    QStringList fields;
                     do
                     {
                         antlr::RefAST indexed_column = tc->getFirstChild();
 
                         QString col = columnname(indexed_column);
-                        FieldPtr field = tab->field(tab->findField(col));
-                        fields.push_back(field);
+                        fields.push_back(col);
 
                         indexed_column = indexed_column->getNextSibling();
                         if(indexed_column != antlr::nullAST
@@ -729,6 +787,7 @@ TablePtr CreateTableWalker::table()
 
                         if(indexed_column != antlr::nullAST && indexed_column->getType() == sqlite3TokenTypes::AUTOINCREMENT)
                         {
+                            auto field = findField(tab, col);
                             field->setAutoIncrement(true);
                             indexed_column = indexed_column->getNextSibling();
                         }
@@ -741,6 +800,10 @@ TablePtr CreateTableWalker::table()
                         }
                     } while(tc != antlr::nullAST && tc->getType() != sqlite3TokenTypes::RPAREN);
 
+                    // We're either done now or there is a conflict clause
+                    tc = tc->getNextSibling();          // skip RPAREN
+                    pk->setConflictAction(parseConflictClause(tc));
+
                     tab->addConstraint(fields, ConstraintPtr(pk));
                 }
                 break;
@@ -751,14 +814,14 @@ TablePtr CreateTableWalker::table()
 
                     tc = tc->getNextSibling(); // skip UNIQUE
                     tc = tc->getNextSibling(); // skip LPAREN
-                    FieldVector fields;
+                    QStringList fields;
                     do
                     {
                         antlr::RefAST indexed_column = tc->getFirstChild();
 
                         QString col = columnname(indexed_column);
-                        FieldPtr field = tab->field(tab->findField(col));
-                        fields.push_back(field);
+                        auto field = findField(tab, col);
+                        fields.push_back(field->name());
 
                         indexed_column = indexed_column->getNextSibling();
                         if(indexed_column != antlr::nullAST
@@ -788,7 +851,7 @@ TablePtr CreateTableWalker::table()
 
                     if(fields.size() == 1 && constraint_name.isEmpty())
                     {
-                        fields[0]->setUnique(true);
+                        findField(tab, fields[0])->setUnique(true);
                         delete unique;
                     } else {
                         tab->addConstraint(fields, ConstraintPtr(unique));
@@ -804,12 +867,11 @@ TablePtr CreateTableWalker::table()
                     tc = tc->getNextSibling();  // KEY
                     tc = tc->getNextSibling();  // LPAREN
 
-                    FieldVector fields;
+                    QStringList fields;
                     do
                     {
                         QString col = columnname(tc);
-                        FieldPtr field = tab->field(tab->findField(col));
-                        fields.push_back(field);
+                        fields.push_back(findField(tab, col)->name());
 
                         tc = tc->getNextSibling();
 
@@ -851,30 +913,13 @@ TablePtr CreateTableWalker::table()
                     tc = tc->getNextSibling(); // skip CHECK
                     tc = tc->getNextSibling(); // skip LPAREN
 
-                    int num_paren = 1;
-                    QString expr;
-                    while(tc)
-                    {
-                        if(tc->getType() == sqlite3TokenTypes::LPAREN)
-                            num_paren++;
-                        else if(tc->getType() == sqlite3TokenTypes::RPAREN)
-                            num_paren--;
-
-                        if(num_paren == 0)
-                            break;
-
-                        expr.append(textAST(tc));
-
-                        tc = tc->getNextSibling();
-                    }
-
-                    check->setExpression(expr);
-                    tab->addConstraint(FieldVector(), ConstraintPtr(check));
+                    check->setExpression(concatExprAST(tc));
+                    tab->addConstraint(QStringList(), ConstraintPtr(check));
                 }
                 break;
                 default:
                 {
-                    qWarning() << "unknown table constraint in " << tab->name();
+                    std::cout << "unknown table constraint in " << tab->name().toStdString() << std::endl;
                     tab->setFullyParsed(false);
                 }
                     break;
@@ -889,7 +934,7 @@ TablePtr CreateTableWalker::table()
                 s = s->getNextSibling();    // WITHOUT
                 s = s->getNextSibling();    // ROWID
 
-                tab->setRowidColumn(tab->fields().at(tab->findPk())->name());
+                tab->setRowidColumn(tab->findPk()->name());
             }
         }
     }
@@ -907,14 +952,35 @@ void CreateTableWalker::parsecolumn(Table* table, antlr::RefAST c)
     QString defaultvalue;
     QString check;
     QString collation;
-    sqlb::PrimaryKeyConstraint* primaryKey = 0;
-    sqlb::ForeignKeyClause* foreignKey = 0;
+    sqlb::PrimaryKeyConstraint* primaryKey = nullptr;
+    std::vector<sqlb::ForeignKeyClause*> foreignKeys;
 
     colname = columnname(c);
     c = c->getNextSibling(); //type?
     if(c != antlr::nullAST && c->getType() == sqlite3TokenTypes::TYPE_NAME)
     {
-        type = concatTextAST(c->getFirstChild(), true);
+        antlr::RefAST t = c->getFirstChild();
+
+        if(t != antlr::nullAST)
+        {
+            type.clear();
+        }
+
+        while(t != antlr::nullAST)
+        {
+            int thisType = t->getType();
+            type.append(textAST(t));
+            t = t->getNextSibling();
+            if(t != antlr::nullAST)
+            {
+                int nextType = t->getType();
+                if(nextType != sqlite3TokenTypes::LPAREN && nextType != sqlite3TokenTypes::RPAREN &&
+                   thisType != sqlite3TokenTypes::LPAREN)
+                {
+                    type.append(" ");
+                }
+            }
+        }
         c = c->getNextSibling();
     }
 
@@ -937,6 +1003,11 @@ void CreateTableWalker::parsecolumn(Table* table, antlr::RefAST c)
         {
         case sqlite3TokenTypes::PRIMARY:
         {
+            // If we have already allocated a primary key object, delete it first. This should never happen in real world situations because
+            // SQLite only supports one primary key constraint per field.
+            if(primaryKey)
+                delete primaryKey;
+
             primaryKey = new PrimaryKeyConstraint;
             primaryKey->setName(constraint_name);
 
@@ -947,6 +1018,9 @@ void CreateTableWalker::parsecolumn(Table* table, antlr::RefAST c)
                 table->setFullyParsed(false);
                 con = con->getNextSibling(); //skip
             }
+
+            primaryKey->setConflictAction(parseConflictClause(con));
+
             if(con != antlr::nullAST && con->getType() == sqlite3TokenTypes::AUTOINCREMENT)
                 autoincrement = true;
         }
@@ -971,12 +1045,10 @@ void CreateTableWalker::parsecolumn(Table* table, antlr::RefAST c)
             if(!constraint_name.isEmpty())
                 table->setFullyParsed(false);
 
+            con = con->getNextSibling(); //CHECK
             con = con->getNextSibling(); //LPAREN
-            check = concatTextAST(con, true);
-            // remove parenthesis
-            check.remove(check.length()-1, 1);
-            check.remove(0,1);
-            check = check.trimmed();
+
+            check = concatExprAST(con);
         }
         break;
         case sqlite3TokenTypes::DEFAULT:
@@ -1002,7 +1074,7 @@ void CreateTableWalker::parsecolumn(Table* table, antlr::RefAST c)
         {
             con = con->getNextSibling();    // REFERENCES
 
-            foreignKey = new ForeignKeyClause;
+            sqlb::ForeignKeyClause* foreignKey = new ForeignKeyClause;
             foreignKey->setTable(identifier(con));
             foreignKey->setName(constraint_name);
             con = con->getNextSibling();    // identifier
@@ -1024,6 +1096,7 @@ void CreateTableWalker::parsecolumn(Table* table, antlr::RefAST c)
             }
 
             foreignKey->setConstraint(concatTextAST(con, true));
+            foreignKeys.push_back(foreignKey);
         }
         break;
         case sqlite3TokenTypes::COLLATE:
@@ -1035,7 +1108,7 @@ void CreateTableWalker::parsecolumn(Table* table, antlr::RefAST c)
         break;
         default:
         {
-            qWarning() << "unknown column constraint in " << table->name() << "." << colname;
+            std::cout << "unknown column constraint in " << table->name().toStdString() << "." << colname.toStdString() << std::endl;
             table->setFullyParsed(false);
         }
         break;
@@ -1043,28 +1116,43 @@ void CreateTableWalker::parsecolumn(Table* table, antlr::RefAST c)
         c = c->getNextSibling();
     }
 
-    FieldPtr f = FieldPtr(new Field(colname, type, notnull, defaultvalue, check, unique, collation));
-    f->setAutoIncrement(autoincrement);
-    table->addField(f);
+    Field f(colname, type, notnull, defaultvalue, check, unique, collation);
+    f.setAutoIncrement(autoincrement);
+    table->fields.push_back(f);
 
-    if(foreignKey)
-        table->addConstraint({f}, ConstraintPtr(foreignKey));
+    for(sqlb::ForeignKeyClause* fk : foreignKeys)
+        table->addConstraint({f.name()}, ConstraintPtr(fk));
     if(primaryKey)
     {
-        FieldVector v;
+        QStringList v;
         if(table->constraint(v, Constraint::PrimaryKeyConstraintType))
         {
-            table->primaryKeyRef().push_back(f);
+            table->primaryKeyRef().push_back(f.name());
 
             // Delete useless primary key constraint. There already is a primary key object for this table, we
             // don't need another one.
             delete primaryKey;
         } else {
-            table->addConstraint({f}, ConstraintPtr(primaryKey));
+            table->addConstraint({f.name()}, ConstraintPtr(primaryKey));
         }
     }
 }
 
+QString CreateTableWalker::parseConflictClause(antlr::RefAST c)
+{
+    QString conflictAction;
+
+    if(c != antlr::nullAST && c->getType() == sqlite3TokenTypes::ON && c->getNextSibling()->getType() == sqlite3TokenTypes::CONFLICT)
+    {
+        c = c->getNextSibling();      // skip ON
+        c = c->getNextSibling();      // skip CONFLICT
+        conflictAction = identifier(c);
+        c = c->getNextSibling();      // skip action
+    }
+
+    return conflictAction;
+}
+
 
 
 QString IndexedColumn::toString(const QString& indent, const QString& sep) const
@@ -1076,7 +1164,6 @@ QString IndexedColumn::toString(const QString& indent, const QString& sep) const
 
 Index::~Index()
 {
-    clear();
 }
 
 Index& Index::operator=(const Index& rhs)
@@ -1090,65 +1177,30 @@ Index& Index::operator=(const Index& rhs)
     m_whereExpr = rhs.m_whereExpr;
 
     // Make copies of the column
-    foreach(IndexedColumnPtr c, rhs.m_columns)
-        addColumn(IndexedColumnPtr(new IndexedColumn(*c)));
+    for(const IndexedColumn& c : rhs.fields)
+        fields.push_back(c);
 
     return *this;
 }
 
-void Index::clear()
-{
-    m_name.clear();
-    m_unique = false;
-    m_table.clear();
-    m_whereExpr.clear();
-    m_columns.clear();
-}
-
-bool Index::removeColumn(const QString& name)
-{
-    int index = findColumn(name);
-    if(index != -1)
-    {
-        m_columns.remove(index);
-        return true;
-    }
-    return false;
-}
-
-void Index::setColumns(const IndexedColumnVector& columns)
-{
-    m_columns = columns;
-}
-
-int Index::findColumn(const QString& name) const
-{
-    for(int i=0;i<m_columns.count();++i)
-    {
-        if(m_columns.at(i)->name().compare(name, Qt::CaseInsensitive) == 0)
-            return i;
-    }
-    return -1;
-}
-
 QStringList Index::columnSqlList() const
 {
     QStringList sl;
 
-    foreach(const IndexedColumnPtr& c, m_columns)
-        sl << c->toString();
+    for(const IndexedColumn& c : fields)
+        sl << c.toString();
 
     return sl;
 }
 
-QString Index::sql(bool ifNotExists) const
+QString Index::sql(const QString& schema, bool ifNotExists) const
 {
     // Start CREATE (UNIQUE) INDEX statement
     QString sql = QString("CREATE %1INDEX%2 %3 ON %4 (\n")
             .arg(m_unique ? QString("UNIQUE ") : QString(""))
             .arg(ifNotExists ? QString(" IF NOT EXISTS") : QString(""))
-            .arg(escapeIdentifier(m_name))
-            .arg(escapeIdentifier(m_table));
+            .arg(ObjectIdentifier(schema, m_name).toString(true))
+            .arg(sqlb::escapeIdentifier(m_table));
 
     // Add column list
     sql += columnSqlList().join(",\n");
@@ -1164,13 +1216,15 @@ QString Index::sql(bool ifNotExists) const
 FieldInfoList Index::fieldInformation() const
 {
     FieldInfoList result;
-    foreach(IndexedColumnPtr c, m_columns)
-        result.append({c->name(), c->order(), c->toString("  ", " ")});
+    for(const IndexedColumn& c : fields)
+        result.emplace_back(c.name(), c.order(), c.toString("  ", " "));
     return result;
 }
 
-ObjectPtr Index::parseSQL(const QString& sSQL)
+IndexPtr Index::parseSQL(const QString& sSQL)
 {
+    SetLocaleToC locale;
+
     std::stringstream s;
     s << sSQL.toStdString();
     Sqlite3Lexer lex(s);
@@ -1186,15 +1240,17 @@ ObjectPtr Index::parseSQL(const QString& sSQL)
         parser.createindex();
         CreateIndexWalker ctw(parser.getAST());
 
-        return ctw.index();
+        auto i = ctw.index();
+        i->setOriginalSql(sSQL);
+        return i;
     }
     catch(antlr::ANTLRException& ex)
     {
-        qCritical() << "Sqlite parse error: " << QString::fromStdString(ex.toString()) << "(" << sSQL << ")";
+        std::cerr << "Sqlite parse error: " << ex.toString() << "(" << sSQL.toStdString() << ")" << std::endl;
     }
     catch(...)
     {
-        qCritical() << "Sqlite parse error: " << sSQL; //TODO
+        std::cerr << "Sqlite parse error: " << sSQL.toStdString() << std::endl; //TODO
     }
 
     return IndexPtr(new Index(""));
@@ -1317,45 +1373,30 @@ void CreateIndexWalker::parsecolumn(Index* index, antlr::RefAST c)
         c = c->getNextSibling();
     }
 
-    index->addColumn(IndexedColumnPtr(new IndexedColumn(name, isExpression, order)));
+    index->fields.emplace_back(name, isExpression, order);
 }
 
 
 
 View::~View()
 {
-    clear();
 }
 
-ObjectPtr View::parseSQL(const QString& /*sSQL*/)
+ViewPtr View::parseSQL(const QString& sSQL)
 {
     // TODO
 
-    return ViewPtr(new View(""));
-}
-
-void View::clear()
-{
-    m_fields.clear();
-}
-
-void View::addField(const FieldPtr& f)
-{
-    m_fields.append(FieldPtr(f));
-}
-
-void View::setFields(const FieldVector& fields)
-{
-    clear();
-    m_fields = fields;
+    auto v = ViewPtr(new View(""));
+    v->setOriginalSql(sSQL);
+    return v;
 }
 
 QStringList View::fieldNames() const
 {
     QStringList sl;
 
-    foreach(FieldPtr f, m_fields)
-        sl << f->name();
+    for(const Field& f : fields)
+        sl << f.name();
 
     return sl;
 }
@@ -1363,18 +1404,19 @@ QStringList View::fieldNames() const
 FieldInfoList View::fieldInformation() const
 {
     FieldInfoList result;
-    foreach(FieldPtr f, m_fields)
-        result.append({f->name(), f->type(), f->toString("  ", " ")});
+    for(const Field& f : fields)
+        result.emplace_back(f.name(), f.type(), f.toString("  ", " "));
     return result;
 }
 
 
-
-ObjectPtr Trigger::parseSQL(const QString& /*sSQL*/)
+TriggerPtr Trigger::parseSQL(const QString& sSQL)
 {
     // TODO
 
-    return TriggerPtr(new Trigger(""));
+    auto t = TriggerPtr(new Trigger(""));
+    t->setOriginalSql(sSQL);
+    return t;
 }
 
 } //namespace sqlb
