@@ -2,24 +2,145 @@
 #ifndef SQLITETYPES_H
 #define SQLITETYPES_H
 
+#include <vector>
+#include <unordered_map>
+#include <memory>
 #include <QString>
-#include <QSharedPointer>
-#include <QVector>
-#include <QStringList>
-#include <QMultiHash>
+#include <QVariant>
+
+template<typename C, typename E>
+bool contains(const C& container, E element)
+{
+    return std::find(container.begin(), container.end(), element) != container.end();
+}
 
 namespace sqlb {
 
 #if QT_VERSION < QT_VERSION_CHECK(5, 6, 0)
+
+#if QT_VERSION < QT_VERSION_CHECK(5, 5, 0)
+struct QHashCombine {
+    typedef uint result_type;
+    template <typename T>
+    Q_DECL_CONSTEXPR result_type operator()(uint seed, const T &t) const Q_DECL_NOEXCEPT_EXPR(noexcept(qHash(t)))
+    // combiner taken from N3876 / boost::hash_combine
+    { return seed ^ (qHash(t) + 0x9e3779b9 + (seed << 6) + (seed >> 2)) ; }
+};
+
+template <typename InputIterator>
+inline uint qHashRange(InputIterator first, InputIterator last, uint seed = 0)
+    Q_DECL_NOEXCEPT_EXPR(noexcept(qHash(*first))) // assume iterator operations don't throw
+{
+    return std::accumulate(first, last, seed, QHashCombine());
+}
+#endif
+
 template<typename T>
-uint qHash(const QVector<T>& key, uint seed = 0)
+uint qHash(const QList<T>& key, uint seed = 0)
     Q_DECL_NOEXCEPT_EXPR(noexcept(qHashRange(key.cbegin(), key.cend(), seed)))
 {
     return qHashRange(key.cbegin(), key.cend(), seed);
 }
 #endif
 
+enum escapeQuoting {
+    DoubleQuotes,
+    GraveAccents,
+    SquareBrackets
+};
+
+// Set quoting style for escapeIdentifier
+void setIdentifierQuoting(escapeQuoting toQuoting);
+
 QString escapeIdentifier(QString id);
+QStringList escapeIdentifier(const QStringList& ids);
+
+class ObjectIdentifier
+{
+public:
+    ObjectIdentifier(const QString& schema, const QString& name)
+        : m_schema(schema),
+          m_name(name)
+    {
+    }
+
+    ObjectIdentifier()
+        : m_schema("main"),
+          m_name(QString())
+    {
+    }
+
+    explicit ObjectIdentifier(QVariant variant)
+    {
+        QStringList str = variant.toStringList();
+        if(str.size())
+        {
+            m_schema = str.first();
+            if(str.size() >= 2)
+                m_name = str.last();
+        }
+    }
+
+    bool operator==(const ObjectIdentifier& rhs) const
+    {
+        return (rhs.m_schema == m_schema && rhs.m_name == m_name);
+    }
+
+    bool operator<(const ObjectIdentifier& rhs) const
+    {
+        return toDisplayString() < rhs.toDisplayString();
+    }
+
+    const QString& schema() const { return m_schema; }
+    const QString& name() const { return m_name; }
+    void setSchema(const QString& schema) { m_schema = schema; }
+    void setName(const QString& name) { m_name = name; }
+
+    void clear()
+    {
+        m_schema = "main";
+        m_name.clear();
+    }
+
+    bool isEmpty() const { return m_name.isEmpty(); }
+
+    // This returns a string which can be used in SQL statements
+    QString toString(bool shortName = false) const
+    {
+        if(shortName && m_schema == "main")
+            return sqlb::escapeIdentifier(m_name);
+        else
+            return QString("%1.%2").arg(sqlb::escapeIdentifier(m_schema)).arg(sqlb::escapeIdentifier(m_name));
+    }
+
+    // This returns a string which can be used in the user interface
+    QString toDisplayString() const
+    {
+        if(m_schema == "main")
+            return m_name;
+        else
+            return QString("%1.%2").arg(m_schema).arg(m_name);
+    }
+
+    QVariant toVariant() const
+    {
+        QStringList result;
+        result << m_schema << m_name;
+        return QVariant(result);
+    }
+
+private:
+    QString m_schema;
+    QString m_name;
+};
+
+struct StringListHash
+{
+    size_t operator()(const QStringList& key) const
+    {
+        return qHash(key);
+    }
+};
 
 class Object;
 class Table;
@@ -30,23 +151,23 @@ class Field;
 class Constraint;
 class IndexedColumn;
 struct FieldInfo;
-typedef QSharedPointer<Object> ObjectPtr;
-typedef QSharedPointer<Table> TablePtr;
-typedef QSharedPointer<Index> IndexPtr;
-typedef QSharedPointer<View> ViewPtr;
-typedef QSharedPointer<Trigger> TriggerPtr;
-typedef QSharedPointer<Field> FieldPtr;
-typedef QSharedPointer<Constraint> ConstraintPtr;
-typedef QVector<FieldPtr> FieldVector;
-typedef QSharedPointer<IndexedColumn> IndexedColumnPtr;
-typedef QVector<IndexedColumnPtr> IndexedColumnVector;
-typedef QMultiHash<FieldVector, ConstraintPtr> ConstraintMap;
-typedef QList<FieldInfo> FieldInfoList;
-
-QStringList fieldVectorToFieldNames(const sqlb::FieldVector& vector);
+using ObjectPtr = std::shared_ptr<Object>;
+using TablePtr = std::shared_ptr<Table>;
+using IndexPtr = std::shared_ptr<Index>;
+using ViewPtr = std::shared_ptr<View>;
+using TriggerPtr = std::shared_ptr<Trigger>;
+using ConstraintPtr = std::shared_ptr<Constraint>;
+using FieldVector = std::vector<Field>;
+using IndexedColumnVector = std::vector<IndexedColumn>;
+using ConstraintMap = std::unordered_multimap<QStringList, ConstraintPtr, StringListHash>;
+using FieldInfoList = std::vector<FieldInfo>;
 
 struct FieldInfo
 {
+    FieldInfo(const QString& name_, const QString& type_, const QString& sql_)
+        : name(name_), type(type_), sql(sql_)
+    {}
+
     QString name;
     QString type;
     QString sql;
@@ -63,7 +184,7 @@ public:
         Trigger
     };
 
-    explicit Object(const QString& name): m_name(name), m_temporary(false), m_fullyParsed(false) {}
+    explicit Object(const QString& name): m_name(name), m_fullyParsed(false) {}
     virtual ~Object() {}
 
     virtual Types type() const = 0;
@@ -75,9 +196,6 @@ public:
     void setOriginalSql(const QString& original_sql) { m_originalSql = original_sql; }
     QString originalSql() const { return m_originalSql; }
 
-    void setTemporary(bool temp) { m_temporary = temp; }
-    bool isTemporary() const { return m_temporary; }
-
     virtual QString baseTable() const { return QString(); }
 
     void setFullyParsed(bool fully_parsed) { m_fullyParsed = fully_parsed; }
@@ -85,27 +203,17 @@ public:
 
     virtual FieldInfoList fieldInformation() const { return FieldInfoList(); }
 
-    virtual void clear() {}
-
     /**
      * @brief Returns the CREATE statement for this object
+     * @param schema The schema name of the object
      * @param ifNotExists If set to true the "IF NOT EXISTS" qualifier will be added to the create statement
      * @return A QString with the CREATE statement.
      */
-    virtual QString sql(bool ifNotExists = false) const = 0;
-
-    /**
-     * @brief parseSQL Parses the CREATE statement in sSQL.
-     * @param sSQL The type of the object.
-     * @param sSQL The create statement.
-     * @return The parsed database object. The object may be empty if parsing failed.
-     */
-    static ObjectPtr parseSQL(Object::Types type, const QString& sSQL);
+    virtual QString sql(const QString& schema = QString("main"), bool ifNotExists = false) const = 0;
 
 protected:
     QString m_name;
     QString m_originalSql;
-    bool m_temporary;
     bool m_fullyParsed;
 };
 
@@ -121,7 +229,7 @@ public:
         CheckConstraintType,
     };
 
-    Constraint(const QString& name = QString())
+    explicit Constraint(const QString& name = QString())
         : m_name(name)
     {
     }
@@ -132,7 +240,7 @@ public:
     void setName(const QString& name) { m_name = name; }
     const QString& name() const { return m_name; }
 
-    virtual QString toSql(const FieldVector& applyOn) const = 0;
+    virtual QString toSql(const QStringList& applyOn) const = 0;
 
 protected:
     QString m_name;
@@ -162,7 +270,7 @@ public:
     void setConstraint(const QString& constraint) { m_constraint = constraint; }
     const QString& constraint() const { return m_constraint; }
 
-    virtual QString toSql(const FieldVector& applyOn) const;
+    virtual QString toSql(const QStringList& applyOn) const;
 
     virtual ConstraintTypes type() const { return ForeignKeyConstraintType; }
 
@@ -179,7 +287,7 @@ class UniqueConstraint : public Constraint
 public:
     UniqueConstraint() {}
 
-    virtual QString toSql(const FieldVector& applyOn) const;
+    virtual QString toSql(const QStringList& applyOn) const;
 
     virtual ConstraintTypes type() const { return UniqueConstraintType; }
 };
@@ -189,15 +297,21 @@ class PrimaryKeyConstraint : public Constraint
 public:
     PrimaryKeyConstraint() {}
 
-    virtual QString toSql(const FieldVector& applyOn) const;
+    void setConflictAction(const QString& conflict) { m_conflictAction = conflict; }
+    const QString& conflictAction() const { return m_conflictAction; }
+
+    virtual QString toSql(const QStringList& applyOn) const;
 
     virtual ConstraintTypes type() const { return PrimaryKeyConstraintType; }
+
+private:
+    QString m_conflictAction;
 };
 
 class CheckConstraint : public Constraint
 {
 public:
-    CheckConstraint(const QString& expr = QString())
+    explicit CheckConstraint(const QString& expr = QString())
         : m_expression(expr)
     {
     }
@@ -205,7 +319,7 @@ public:
     void setExpression(const QString& expr) { m_expression = expr; }
     QString expression() const { return m_expression; }
 
-    virtual QString toSql(const FieldVector& applyOn) const;
+    virtual QString toSql(const QStringList& applyOn) const;
 
     virtual ConstraintTypes type() const { return CheckConstraintType; }
 
@@ -216,6 +330,12 @@ private:
 class Field
 {
 public:
+    Field()
+        : m_notnull(false),
+          m_autoincrement(false),
+          m_unique(false)
+    {}
+
     Field(const QString& name,
           const QString& type,
           bool notnull = false,
@@ -233,6 +353,11 @@ public:
         , m_collation(collation)
     {}
 
+    bool operator==(const Field& rhs) const
+    {
+        return m_name == rhs.name();
+    }
+
     QString toString(const QString& indent = "\t", const QString& sep = "\t") const;
 
     void setName(const QString& name) { m_name = name; }
@@ -246,6 +371,12 @@ public:
 
     bool isText() const;
     bool isInteger() const;
+    bool isBlob() const;
+    bool isReal() const;
+    bool isNumeric() const;
+
+    // Type affinity of the column according to SQLite3 rules
+    QString affinity() const;
 
     const QString& name() const { return m_name; }
     const QString& type() const { return m_type; }
@@ -256,7 +387,6 @@ public:
     bool unique() const { return m_unique; }
     const QString& collation() const { return m_collation; }
 
-    static QStringList Datatypes;
 private:
     QString m_name;
     QString m_type;
@@ -277,19 +407,16 @@ public:
 
     virtual Types type() const { return Object::Table; }
 
-    const FieldVector& fields() const { return m_fields; }
+    FieldVector fields;
+    using field_type = Field;
+    using field_iterator = FieldVector::iterator;
 
     /**
      * @brief Returns the CREATE TABLE statement for this table object
      * @return A QString with the CREATE TABLE object.
      */
-    QString sql(bool ifNotExists = false) const;
+    QString sql(const QString& schema = QString("main"), bool ifNotExists = false) const;
 
-    void addField(const FieldPtr& f);
-    bool removeField(const QString& sFieldName);
-    void setFields(const FieldVector& fields);
-    void setField(int index, FieldPtr f);
-    const FieldPtr& field(int index) const { return m_fields[index]; }
     QStringList fieldNames() const;
 
     void setRowidColumn(const QString& rowid) {  m_rowidColumn = rowid; }
@@ -300,42 +427,33 @@ public:
     QString virtualUsing() const { return m_virtual; }
     bool isVirtual() const { return !m_virtual.isEmpty(); }
 
-    void clear();
-
     virtual FieldInfoList fieldInformation() const;
 
-    void addConstraint(FieldVector fields, ConstraintPtr constraint);
-    void setConstraint(FieldVector fields, ConstraintPtr constraint);
-    void removeConstraints(FieldVector fields = FieldVector(), Constraint::ConstraintTypes type = Constraint::NoType); //! Only removes the first constraint, if any
-    ConstraintPtr constraint(FieldVector fields = FieldVector(), Constraint::ConstraintTypes type = Constraint::NoType) const;   //! Only returns the first constraint, if any
-    QList<ConstraintPtr> constraints(FieldVector fields = FieldVector(), Constraint::ConstraintTypes type = Constraint::NoType) const;
+    void addConstraint(QStringList fields, ConstraintPtr constraint);
+    void setConstraint(QStringList fields, ConstraintPtr constraint);
+    void removeConstraints(QStringList fields = QStringList(), Constraint::ConstraintTypes type = Constraint::NoType); //! Only removes the first constraint, if any
+    ConstraintPtr constraint(QStringList fields = QStringList(), Constraint::ConstraintTypes type = Constraint::NoType) const;   //! Only returns the first constraint, if any
+    std::vector<ConstraintPtr> constraints(QStringList fields = QStringList(), Constraint::ConstraintTypes type = Constraint::NoType) const;
     ConstraintMap allConstraints() const { return m_constraints; }
     void setConstraints(const ConstraintMap& constraints);
-    FieldVector& primaryKeyRef();
-    const FieldVector& primaryKey() const;
+    QStringList& primaryKeyRef();
+    const QStringList& primaryKey() const;
+    void removeKeyFromAllConstraints(const QString& key);
+    void renameKeyInAllConstraints(const QString& key, const QString& to);
 
-    /**
-     * @brief findField Finds a field and returns the index.
-     * @param sname
-     * @return The field index if the field was found.
-     *         -1 if field couldn't be found.
-     */
-    int findField(const QString& sname) const;
-
-    int findPk() const;
+    field_iterator findPk();
 
     /**
      * @brief parseSQL Parses the create Table statement in sSQL.
      * @param sSQL The create table statement.
      * @return The table object. The table object may be empty if parsing failed.
      */
-    static ObjectPtr parseSQL(const QString& sSQL);
+    static TablePtr parseSQL(const QString& sSQL);
 private:
     QStringList fieldList() const;
     bool hasAutoIncrement() const;
 
 private:
-    FieldVector m_fields;
     QString m_rowidColumn;
     ConstraintMap m_constraints;
     QString m_virtual;
@@ -377,6 +495,10 @@ public:
 
     virtual Types type() const { return Object::Index; }
 
+    IndexedColumnVector fields;
+    using field_type = IndexedColumn;
+    using field_iterator = IndexedColumnVector::iterator;
+
     virtual QString baseTable() const { return m_table; }
 
     void setUnique(bool unique) { m_unique = unique; }
@@ -388,29 +510,18 @@ public:
     void setWhereExpr(const QString& expr) { m_whereExpr = expr; }
     const QString& whereExpr() const { return m_whereExpr; }
 
-    void setColumns(const IndexedColumnVector& columns);
-    const IndexedColumnVector& columns() const { return m_columns; }
-    void clearColumns() { m_columns.clear(); }
-    void addColumn(const IndexedColumnPtr& c) { m_columns.append(c); }
-    bool removeColumn(const QString& name);
-    void setColumn(int index, IndexedColumnPtr c) { m_columns[index] = c; }
-    IndexedColumnPtr& column(int index) { return m_columns[index]; }
-    int findColumn(const QString& name) const;
-
-    void clear();
-
     /**
      * @brief Returns the CREATE INDEX statement for this index object
      * @return A QString with the CREATE INDEX object.
      */
-    QString sql(bool ifNotExists = false) const;
+    QString sql(const QString& schema = QString("main"), bool ifNotExists = false) const;
 
     /**
      * @brief parseSQL Parses the CREATE INDEX statement in sSQL.
      * @param sSQL The create index statement.
      * @return The index object. The index object may be empty if the parsing failed.
      */
-    static ObjectPtr parseSQL(const QString& sSQL);
+    static IndexPtr parseSQL(const QString& sSQL);
 
     virtual FieldInfoList fieldInformation() const;
 
@@ -420,7 +531,6 @@ private:
     bool m_unique;
     QString m_table;
     QString m_whereExpr;
-    IndexedColumnVector m_columns;
 };
 
 class View : public Object
@@ -431,21 +541,15 @@ public:
 
     virtual Types type() const { return Object::View; }
 
-    QString sql(bool ifNotExists = false) const { /* TODO */ Q_UNUSED(ifNotExists); return m_originalSql; }
+    FieldVector fields;
 
-    static ObjectPtr parseSQL(const QString& sSQL);
+    QString sql(const QString& schema = QString("main"), bool ifNotExists = false) const { /* TODO */ Q_UNUSED(schema); Q_UNUSED(ifNotExists); return m_originalSql; }
 
-    void clear();
+    static ViewPtr parseSQL(const QString& sSQL);
 
-    void addField(const FieldPtr& f);
-    void setFields(const FieldVector& fields);
-    const FieldPtr& field(int index) const { return m_fields[index]; }
     QStringList fieldNames() const;
 
     virtual FieldInfoList fieldInformation() const;
-
-private:
-    FieldVector m_fields;
 };
 
 class Trigger : public Object
@@ -456,9 +560,9 @@ public:
 
     virtual Types type() const { return Object::Trigger; }
 
-    QString sql(bool ifNotExists = false) const { /* TODO */ Q_UNUSED(ifNotExists); return m_originalSql; }
+    QString sql(const QString& schema = QString("main"), bool ifNotExists = false) const { /* TODO */ Q_UNUSED(schema); Q_UNUSED(ifNotExists); return m_originalSql; }
 
-    static ObjectPtr parseSQL(const QString& sSQL);
+    static TriggerPtr parseSQL(const QString& sSQL);
 
     virtual QString baseTable() const { return m_table; }
 
@@ -469,6 +573,60 @@ private:
     QString m_table;
 };
 
+/**
+ * @brief findField Finds a field in the database object and returns an iterator to it.
+ * @param object
+ * @param name
+ * @return The iterator pointing to the field in the field container of the object if the field was found.
+ *         object.fields.end() if the field couldn't be found.
+ */
+template<typename T>
+typename T::field_iterator findField(T* object, const QString& name)
+{
+    return std::find_if(object->fields.begin(), object->fields.end(), [&name](const typename T::field_type& f) {return f.name().compare(name, Qt::CaseInsensitive) == 0;});
+}
+template<typename T>
+typename T::field_iterator findField(std::shared_ptr<T> object, const QString& name)
+{
+    return findField(object.get(), name);
+}
+template<typename T>
+typename T::field_iterator findField(T& object, const QString& name)
+{
+    return findField(&object, name);
+}
+
+template<typename T> struct is_shared_ptr : std::false_type {};
+template<typename T> struct is_shared_ptr<std::shared_ptr<T>> : std::true_type {};
+
+/**
+ * @brief removeField Finds and removes a field in the database object
+ * @param object
+ * @param name
+ * @return true if sucessful, otherwise false
+ */
+template<typename T>
+bool removeField(T* object, const QString& name)
+{
+    auto index = findField(object, name);
+    if(index != object->fields.end())
+    {
+        object->fields.erase(index);
+        return true;
+    }
+    return false;
+}
+template<typename T, typename = typename std::enable_if<is_shared_ptr<T>::value>::type>
+bool removeField(T object, const QString& name)
+{
+    return removeField(object.get(), name);
+}
+template<typename T, typename = typename std::enable_if<!is_shared_ptr<T>::value>::type>
+bool removeField(T& object, const QString& name)
+{
+    return removeField(&object, name);
+}
+
 } //namespace sqlb
 
-#endif // SQLITETYPES_H
+#endif
