@@ -4,14 +4,13 @@
 #include <QtTest/QTest>
 #include <QCoreApplication>
 #include <QTextStream>
+#include <QVector>
 
 #include "csvparser.h"
 #include "TestImport.h"
 
 QTEST_MAIN(TestImport)
 
-Q_DECLARE_METATYPE(CSVParser::TCSVResult)
-
 TestImport::TestImport()
 {
 }
@@ -28,15 +27,15 @@ void TestImport::csvImport()
     QFETCH(char, quote);
     QFETCH(QString, encoding);
     QFETCH(int, numfields);
-    QFETCH(QVector<QStringList>, result);
+    QFETCH(QVector<QVector<QByteArray>>, result);
 
     // Create temporary CSV file
     QTemporaryFile file;
     QVERIFY(file.open());
     {
-    QTextStream out(&file);
-    out.setCodec(encoding.toUtf8());
-    out << csv;
+        QTextStream out(&file);
+        out.setCodec(encoding.toUtf8());
+        out << csv;
     }
     file.flush();
 
@@ -44,11 +43,29 @@ void TestImport::csvImport()
     file.seek(0);
     QTextStream tstream(&file);
     tstream.setCodec(encoding.toUtf8());
-    csvparser.parse(tstream);
+
+    QVector<QVector<QByteArray>> parsedCsv;
+    int parsedCsvColumns = 0;
+    csvparser.parse([&parsedCsv, &parsedCsvColumns](size_t /*rowNum*/, const CSVRow& data) -> bool {
+        QVector<QByteArray> row;
+        for(size_t i=0;i<data.num_fields;i++)
+            row.push_back(QByteArray(data.fields[i].data, data.fields[i].data_length));
+        parsedCsv.push_back(row);
+        if(row.size() > parsedCsvColumns)
+            parsedCsvColumns = row.size();
+        return true;
+    }, tstream);
 
     // Check return values
-    QCOMPARE(csvparser.csv(), result);
-    QCOMPARE((int)csvparser.columns(), numfields);
+    QCOMPARE(parsedCsvColumns, numfields);
+    QCOMPARE(parsedCsv.size(), result.size());
+    for(int i=0;i<parsedCsv.size();i++)
+    {
+        QCOMPARE(parsedCsv.at(i).size(), result.at(i).size());
+        for(int j=0;j<parsedCsv.at(i).size();j++)
+            QCOMPARE(parsedCsv.at(i).at(j), result.at(i).at(j));
+    }
+    QCOMPARE(parsedCsv, result);
 }
 
 void TestImport::csvImport_data()
@@ -58,12 +75,12 @@ void TestImport::csvImport_data()
     QTest::addColumn<char>("quote");
     QTest::addColumn<QString>("encoding");
     QTest::addColumn<int>("numfields");
-    QTest::addColumn<CSVParser::TCSVResult>("result");
+    QTest::addColumn<QVector<QVector<QByteArray>>>("result");
 
-    CSVParser::TCSVResult result;
-    result.append(QStringList() << "a" << "b" << "c");
-    result.append(QStringList() << "d" << "e" << "f");
-    result.append(QStringList() << "g" << "h" << "i");
+    QVector<QVector<QByteArray>> result;
+    result.append(QVector<QByteArray>() << "a" << "b" << "c");
+    result.append(QVector<QByteArray>() << "d" << "e" << "f");
+    result.append(QVector<QByteArray>() << "g" << "h" << "i");
     QTest::newRow("commas_noquotes") << "a,b,c\nd,e,f\ng,h,i\n"
                                      << ','
                                      << (char)0
@@ -102,11 +119,11 @@ void TestImport::csvImport_data()
                                        << result;
 
     result.clear();
-    result.append(QStringList() << "a" << "b" << "");
-    result.append(QStringList() << "c" << "");
-    result.append(QStringList() << "d" << "" << "e");
-    result.append(QStringList() << "");
-    result.append(QStringList() << "" << "" << "f");
+    result.append(QVector<QByteArray>() << "a" << "b" << "");
+    result.append(QVector<QByteArray>() << "c" << "");
+    result.append(QVector<QByteArray>() << "d" << "" << "e");
+    result.append(QVector<QByteArray>() << "");
+    result.append(QVector<QByteArray>() << "" << "" << "f");
     QTest::newRow("emptyvalues") << "a,b,\nc,\nd,,e\n\n,,f"
                                      << ','
                                      << (char)0
@@ -115,7 +132,7 @@ void TestImport::csvImport_data()
                                      << result;
 
     result.clear();
-    result.append(QStringList() << "a" << "b" << "c");
+    result.append(QVector<QByteArray>() << "a" << "b" << "c");
     QTest::newRow("oneline") << "a,b,c"
                              << ','
                              << (char)0
@@ -124,8 +141,8 @@ void TestImport::csvImport_data()
                              << result;
 
     result.clear();
-    result.append(QStringList() << "a,a\"" << "b" << "c");
-    result.append(QStringList() << "d" << "e" << "\"\"f,f");
+    result.append(QVector<QByteArray>() << "a,a\"" << "b" << "c");
+    result.append(QVector<QByteArray>() << "d" << "e" << "\"\"f,f");
     QTest::newRow("manyquotes") << "\"a,a\"\"\",\"b\",\"c\"\n\"d\",\"e\",\"\"\"\"\"f,f\"\n"
                                 << ','
                                 << '"'
@@ -134,7 +151,7 @@ void TestImport::csvImport_data()
                                 << result;
 
     result.clear();
-    result.append(QStringList() << QString::fromUtf8("\xC2\xAE") << QString::fromUtf8("\xC9\x85") << QString::fromUtf8("\xC6\x89"));
+    result.append(QVector<QByteArray>() << QByteArray("\xC2\xAE") << QByteArray("\xC9\x85") << QByteArray("\xC6\x89"));
     QString csv = QString::fromUtf8("\xC2\xAE") + "," + QString::fromUtf8("\xC9\x85") + "," + QString::fromUtf8("\xC6\x89") + "\n";
     QTest::newRow("utf8chars") << csv
                                << ','
@@ -144,7 +161,7 @@ void TestImport::csvImport_data()
                                << result;
 
     result.clear();
-    result.append(QStringList() << QString::fromUtf8("\u4E18") << QString::fromUtf8("\u4E26") << QString::fromUtf8("\u4E4B"));
+    result.append(QVector<QByteArray>() << QByteArray("\u4E18") << QByteArray("\u4E26") << QByteArray("\u4E4B"));
     QString csv2 = QString::fromUtf8("\u4E18") + "," + QString::fromUtf8("\u4E26") + "," + QString::fromUtf8("\u4E4B") + "\n";
     QTest::newRow("utf16chars") << csv2
                                << ','
